// 3_gallery.js
// This file retrieves all images available with their sampletext, if available

// Includes
var request = require('request');
var http = require('http');
var fs = require('fs');
var crypto = require('crypto');
http.globalAgent.maxSockets = 1000;

// Configuration
var api_url = 'http://v3.alphapicture.com';
var api_username = 'apiuser';
var api_password = 'password';
var api_key = 'secretkey';

var concurrency = 10;

// Code

var motifList = [];
var motifInfo = JSON.parse(fs.readFileSync('motifs.json'));
// assumes motiflist from first example
for (var i = 0; i < motifInfo.length; i++) {
	for (var j = 0; j < motifInfo[i]['alternatives'].length; j++) {
		motifList.push({
			motifId : motifInfo[i]['id'],
			altId : motifInfo[i]['alternatives'][j]['alternative_id'],
			sampleText : motifInfo[i]['alternatives'][j]['sample_text']
		});
	}
}

function Thread(id) {
	this.id = id;
	this.next();
}

Thread.prototype = {
	next : function() {
		if (motifList.length > 0) {
			var motifToFetch = motifList.shift();
			var requestBody = createRequestBody(motifToFetch['motifId'], motifToFetch['altId'], motifToFetch['sampleText']);
			request({
				uri : api_url + '/Image',
				method : "POST",
				headers : {
					'APIV3-Account' : 'koen',
					'APIV3-Password' : 'password2',
					'APIV3-Signature' : generateSignature(api_key, requestBody)
				},
				json : requestBody,
				timeout : 60000
			}, (function(motifToFetch, error, response, body) {
				//console.log(this.id, 'CALLBACK')
				if (!error && response.statusCode == 200 && typeof body != 'object') {
					var filename = '3_' + motifToFetch['motifId'] + '-' + motifToFetch['altId'] + '_' + (+new Date()) + '.jpg';
					fs.writeFile('./output/' + filename, body, {
						encoding : 'binary'
					}, function(err) {
						if (err) {
							console.log('Save error', motifToFetch['motifId'], motifToFetch['altId'], err);
						} else {
							//console.log("Result image saved to output/" + filename);
						}
					});
				} else {
					console.log('Error retrieving motif ', motifToFetch['motifId'], motifToFetch['altId'], error, JSON.stringify(body));
					//process.exit();
				}
				this.next();
			}).bind(this, motifToFetch));
		}
	}
}

for (var i = 0; i < concurrency; i++) {
	new Thread(i);
}

function createRequestBody(motifId, altId, sampleLines) {
	return {
		"MotifId" : motifId,
		"AlternativeId" : altId,
		"Lines" : sampleLines,
		"Dimensions" : {
			"w" : 1000,
			"h" : 650
		}
	};
}

function generateSignature(key, data) {
	var bodyHash;
	if ( typeof data == 'object') {
		bodyHash = crypto.createHash('sha512').update(JSON.stringify(data)).digest("hex");
	} else {
		bodyHash = crypto.createHash('sha512').update(data).digest("hex");
	}
	return crypto.createHash('sha512').update(bodyHash + key).digest("hex");
}
