<?php

error_reporting(0);
ini_set('display_errors', true);

$api_url = 'http://v3.alphapicture.com';
$api_username = 'username';
$api_password = 'password';
$api_secret = 'secret';

$template = [
	"Images" => [
			[
				"MotifId" => 4,
				"AlternativeId" => 1,
				"Template" => [
					"1+2+3" => "Hello %firstname%"
				],
				"Filename" => "4_%number%",
				"Dimensions" => [
					"w" => 1000,
					"h" => 750
				],
				"Watermark" => false,
				"BadWordFilter" => false
			]
	],
	"OutputOptions" => [
		"OutputMethod" => "HOTLINK"
	],
	"Data" => [
		["firstname" => "AlphaPicture", "number" => 1],
		["firstname" => "AlphaPicture", "number" => 2]
	]
];

$jobId = submitJob($template, $api_url, $api_username, $api_password, $api_secret);

$done = false;
$maxIterations = 1000;
$startTime = time();
$images = [];

while ($done == false) {
	$result = checkJob($jobId, $api_url, $api_username, $api_password, $api_secret);
	if ($result->Status == 'DONE') {
		foreach ($result->Filenames as $f) {
			$images[$f] = $result->CDN . '/' . $f . '.jpg';
		}
		$done = true;
	}
	sleep(5);
	if (time() > $startTime + 300) {
		echo "Timeout occured \n";
		$done = true;
	}
}
echo "Retrieving files one by one...\n";
foreach ($images as $k => $v) {
	echo ".";
	file_put_contents("./".$k.".jpg", file_get_contents($v));
}
echo "\nDone!\n";


function submitJob($template, $api_url, $api_username, $api_password, $api_secret) {
	$templateJson = json_encode($template);
	$ch = curl_init($api_url . "/Job");
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	    'Content-Type: application/json',
	    'Content-Length: ' . strlen($templateJson),
		'APIV3-Account: ' . $api_username,
		'APIV3-Password: ' . $api_password,
		'APIV3-Signature: ' . generateSignature($api_secret, $templateJson)
		)
	);
    curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $templateJson);
	curl_setopt($ch, CURLOPT_TIMEOUT, 15);

	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$output = curl_exec($ch);
	curl_close($ch);

	$outputObj = json_decode($output);

	print_r($outputObj);

	return $outputObj->JobId;
}

function checkJob($id, $api_url, $api_username, $api_password, $api_secret) {
	$template = array();
	$template['Id'] = $id;
	$template['Filenames'] = true;

	$templateJson = json_encode($template);

	$ch = curl_init($api_url . "/JobInfo");
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	    'Content-Type: application/json',
	    'Content-Length: ' . strlen($templateJson),
		'APIV3-Account: ' . $api_username,
		'APIV3-Password: ' . $api_password,
		'APIV3-Signature: ' . generateSignature($api_secret, $templateJson)
		)
	);
    curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $templateJson);
	curl_setopt($ch, CURLOPT_TIMEOUT, 5);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$output = curl_exec($ch);
	curl_close($ch);
	$outputObj = json_decode($output);

	print_r($outputObj);

	return $outputObj;
}

function generateSignature($key, $data) {
	$bodyHash = null;
	if (gettype($data) == 'object' || gettype($data) == 'array') {
		$bodyHash = hash('sha512',json_encode($data),false);
	} else {
		$bodyHash = hash('sha512',$data, false);
	}
	return hash('sha512',$bodyHash . $key, false);
}

?>
